(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6324,430],{7614:function(n,e,r){(window.__NEXT_P=window.__NEXT_P||[]).push(["/workspaces/[name]",function(){return r(9431)}])},9431:function(n,e,r){"use strict";r.r(e),r.d(e,{default:function(){return i}});var u=r(5893);r(7294);var t=r(1163),l=r(1141);function i(){let n=(0,t.useRouter)(),{name:e}=n.query;return n.isReady&&e?(0,u.jsx)(l.WorkspaceEditor,{workspaceName:e}):(0,u.jsx)("div",{children:"Loading..."})}},9008:function(n,e,r){n.exports=r(7219)},1163:function(n,e,r){n.exports=r(6036)},2003:function(n,e,r){"use strict";r.d(e,{j:function(){return i}});var u=r(512);let t=n=>"boolean"==typeof n?`${n}`:0===n?"0":n,l=u.W,i=(n,e)=>r=>{var u;if((null==e?void 0:e.variants)==null)return l(n,null==r?void 0:r.class,null==r?void 0:r.className);let{variants:i,defaultVariants:s}=e,o=Object.keys(i).map(n=>{let e=null==r?void 0:r[n],u=null==s?void 0:s[n];if(null===e)return null;let l=t(e)||t(u);return i[n][l]}),c=r&&Object.entries(r).reduce((n,e)=>{let[r,u]=e;return void 0===u||(n[r]=u),n},{});return l(n,o,null==e?void 0:null===(u=e.compoundVariants)||void 0===u?void 0:u.reduce((n,e)=>{let{class:r,className:u,...t}=e;return Object.entries(t).every(n=>{let[e,r]=n;return Array.isArray(r)?r.includes({...s,...c}[e]):({...s,...c})[e]===r})?[...n,r,u]:n},[]),null==r?void 0:r.class,null==r?void 0:r.className)}}},function(n){n.O(0,[616,5230,5739,1664,804,1272,3947,6989,3698,9470,6990,8969,1043,6601,938,1141,2888,9774,179],function(){return n(n.s=7614)}),_N_E=n.O()}]);